/* --- Constant Variables --- */
// Scroll Types
const WINDOW = 1;
const TEXTBOX = 2;

// Pref Types
const BOOL = 0;
const INT  = 1;
const CHAR = 2;
/* -------------------------- */

/* --- Global Variables ------*/
var cmdDispatcher = null;
var scrollTarget = null;
var scrollType = null;
var scrollbarRegExp = /^xul:(:?slider|thumb)/;
var inputboxRegExp = /^textarea|textbox/i;
var modKeys = new Array('altKey', 'ctrlKey', 'shiftKey', 'metaKey');
var modKey = 0;
var prefs = Components.classes["@mozilla.org/preferences-service;1"].
				getService(Components.interfaces.nsIPrefBranch);

var xsModKey;    // modifier key 0 = alt, 1 = ctrl, 2 = shift, 3 = meta
var xsDelta;     // the amount of movements per action.
var xsDirection; // 0 = up/down => left/right, 1 = up/down => right/left
/* --------------------------*/

function xsInit()
{
	xsModKey = getPref("extensions.xscroll.modkey", INT, 2, 0, modKeys.length-1);
	xsDelta = getPref("extensions.xscroll.delta", INT, 36, 1, 360);
	xsDirection = getPref("extensions.xscroll.direction", INT, 0, 0, 1);

	modKey = modKeys[xsModKey];
	cmdDispatcher = document.commandDispatcher;
	window.addEventListener("DOMMouseScroll", xsEventMouseScroll, false);
}

function getPref(aName, aType, aDefault, aMin, aMax)
{
	var retValue = null;
	switch (aType) {
	case BOOL:
		if (prefs.getPrefType(aName) == prefs.PREF_BOOL) {
			retValue = prefs.getBoolPref(aName);
		} else {
			prefs.setBoolPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	case INT:
		if (prefs.getPrefType(aName) == prefs.PREF_INT) {
			retValue = prefs.getIntPref(aName);
			if (retValue < aMin || retValue > aMax)
				retValue = aDefault;
		} else {
			prefs.setIntPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	case CHAR:
		if (prefs.getPrefType(aName) == prefs.PREF_STRING) {
			retValue = prefs.getCharPref(aName);
		} else {
			prefs.setCharPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	}
	return retValue;	
}

function xsEventMouseScroll(aEvent)
{
	if ( isXScrollable(aEvent) ) {
		doXScroll(aEvent, scrollType);
	} else if ( (aEvent[modKey] || cmdDispatcher.focusedElement) &&
		 isTextBox(aEvent) ) {
		doXScroll(aEvent, scrollType); 
	} else if ( aEvent[modKey] ) {
		scrollTarget = cmdDispatcher.focusedWindow;
		scrollType = WINDOW;
		doXScroll(aEvent, scrollType); 
	}
}

function isTextBox(aEvent) {
    if (!aEvent.target.tagName ||
		!aEvent[modKey] && aEvent.target.tagName == 'TEXTAREA')
		return false;

    if (aEvent.target.tagName.match(inputboxRegExp)) {
		scrollTarget = cmdDispatcher.focusedElement;
		scrollType = TEXTBOX;
		return true;
    }
}

function isXScrollable(aEvent) {
	if (aEvent.originalTarget.tagName &&
		aEvent.originalTarget.tagName.match(scrollbarRegExp) &&
		aEvent.originalTarget.getAttribute("orient") == 'horizontal') {

		if (aEvent.target.tagName == 'TEXTAREA') {
			if (cmdDispatcher.focusedElement) {
				scrollTarget = cmdDispatcher.focusedElement;
				scrollType = TEXTBOX;
			} else return false;
		} else {
			scrollTarget = cmdDispatcher.focusedWindow;
			scrollType = WINDOW;
		}
		return true;
    }
	return false;
}

function doXScroll(aEvent, aType)
{
	var detail = aEvent.detail;
    if (xsDirection) detail *= -1;
	if (aType == WINDOW) {
		if (detail > 0)
			scrollTarget.scrollBy(xsDelta, 0);
		else
			scrollTarget.scrollBy(-xsDelta, 0);
    } else {
		if (detail > 0)
			scrollTarget.scrollLeft += xsDelta;
		else
			scrollTarget.scrollLeft -= xsDelta;
    }
    aEvent.preventDefault();
}

